import ij.*;
import ij.plugin.*;
import ij.plugin.filter.*;
import ij.process.*;
import ij.gui.*;

/**	This is an extended ParticleAnalyzer that creates a copy of the
	image that has each measured particle cleared. For stacks, the
	particles are cleared in place.
*/
public class Particle_Remover implements PlugIn {

	public void run(String arg) {
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp==null || imp.getType()!=ImagePlus.GRAY8)
			{IJ.error("8-bit grayscale image required"); return;}
		ImageStack stack = imp.getStack();
		int stackSize = imp.getStackSize();
		CustomAnalyzer pa = new CustomAnalyzer();
		pa.setup("", imp);
		for (int i=1; i<=stackSize; i++) {
			IJ.showProgress((double)i/stackSize);
			IJ.showStatus("a: "+i+"/"+stackSize);
			imp.setSlice(i);
			ImageProcessor ip = imp.getProcessor();
			pa.run(imp, ip, i);
			if (pa.error)
				break;
			if (stackSize==1)
				new ImagePlus(imp.getTitle()+"2", pa.ip2).show();
			else
				ip.setPixels(pa.ip2.getPixels());
		}
		IJ.showProgress(1.0);
	}
	
}

class CustomAnalyzer extends ParticleAnalyzer {
	public ImageProcessor ip2;
	public boolean error;
	
	public void run(ImagePlus imp, ImageProcessor ip, int slice) {
		ip.setRoi(null);
		ip2 = ip.crop(); // make a copy of this image
		ip2.setValue(0); // fill value
		if (slice==1)
			super.run(ip); //shows dialog
		else
			error = !analyze(imp, ip); //no dialog
	}
	
	protected void saveResults(ImageStatistics stats, Roi roi) {
		super.saveResults(stats, roi);
		ip2.setRoi(roi.getBoundingRect());
		ip2.fill(roi.getMask());
	}
}
