import ij.*;
import ij.plugin.filter.*;
import ij.process.*;
import ij.gui.*;
import ij.measure.*;
import java.awt.*;

/** This sample plugin illustrates how to extend ImageJ's ParticleAnalyzer class.
	For each particle, it displays the particle number, the XY coordinates of the
	first pixel encountered in the outlining process, and the number of
	XY coordinates in the particle outline.
*/

public class Particle_Analyzer implements PlugInFilter, Measurements {
	ImagePlus imp;

	public int setup(String arg, ImagePlus imp) {
		this.imp = imp;
		return DOES_8G;
	}

	public void run(ImageProcessor ip) {
		if (IJ.versionLessThan("1.17g"))
			return;
		int options = ParticleAnalyzer.SHOW_PROGRESS;
		int measurements = CENTROID;
		ResultsTable table = new ResultsTable();
		int minSize=1, maxSize=999999;
		ParticleAnalyzer pa = new MyParticleAnalyzer(options, measurements, table, minSize, maxSize);
		pa.analyze(imp);
	}

}

class MyParticleAnalyzer extends ParticleAnalyzer {

	int n;

	public MyParticleAnalyzer(int options, int measurements, ResultsTable rt, int min, int max) {
		super(options, measurements, rt, min, max);
		IJ.setColumnHeadings("N\tX Start\tY Start\tXY Coordinates");
	}
	
	// Overrides method with the same in AnalyzeParticles that's called once for each particle
	protected void saveResults(ImageStatistics stats, Roi roi) {
		super.saveResults(stats, roi);
		int coordinates = ((PolygonRoi)roi).getNCoordinates();
		Rectangle r = roi.getBoundingRect();
		int x = r.x+((PolygonRoi)roi).getXCoordinates()[coordinates-1];
		int y = r.y+((PolygonRoi)roi).getYCoordinates()[coordinates-1];
		IJ.write(++n+"\t"+x+"\t"+y+"\t"+coordinates);
	}
}


